@@ -7,6 +7,7 @@ import psutil
 import time
 import hashlib
 import json
+import time
 
 import logging
 import logging.handlers
@@ -16,7 +17,7 @@ import evalidate
 import socket
 from socket import AF_INET, SOCK_STREAM, SOCK_DGRAM
 
-version='1.9.23 (datetime)'
+version='2.0.15 (daemon)'
 
 
 # TODO
@@ -25,6 +26,36 @@ version='1.9.23 (datetime)'
 # --prefix
 
 
+class OkerrExc(Exception):
+    pass
+
+class OkerrClientServer(OkerrExc):
+    pass
+
+    
+class OkerrKeyAuth(OkerrExc):
+    pass
+        
+class OkerrNoKey(OkerrExc):
+    pass
+        
+class OkerrNoTextID(OkerrExc):
+    pass
+    
+class OkerrBadData(OkerrExc):
+    pass
+
+class OkerrBadMethod(OkerrExc):
+    pass
+
+
+
+def briefpage(s, sz=50):
+    for ch in '\r\n\t':
+        s = s.replace(ch,'.')
+    return s
+
+
 class OkerrClient:
 
     # TODO: implement list of caches including ~/.okerr-cache.json
@@ -35,6 +66,8 @@ class OkerrClient:
     log = None
     cache = None   
     prefix = None    
+    
+    
         
     cfgvars = ['secret','textid','url','keyuser','keypass','cachepath']
     
@@ -43,6 +76,10 @@ class OkerrClient:
         # default null log handler
         self.log = logging.getLogger()
         self.log.addHandler(logging.NullHandler())
+        
+        self.retry = False # default
+        self.sleeptime = 10
+        
         if cfg:            
             for k in self.cfgvars:
                 if k in cfg:
@@ -63,6 +100,7 @@ class OkerrClient:
                 textid=self.textid, 
                 secret=self.secret)    
 
+
     def load(self):
         self.loadcache()
 
@@ -102,36 +140,74 @@ class OkerrClient:
             self.log.debug('cannot save cache {}'.format(self.cachepath))
             print(self.cache)
     
+    
+    def setretry(self,retry):
+        self.retry = retry
+    
+    def altkeypath(self,path):
+        ntried=0
+        
+        for trypath in path.split('|'):
+            try:
+                ntried+=1
+                self.log.debug('try key {}'.format(trypath))
+                data = self.keypath(trypath)
+                if data is not None:
+                    return (data, trypath)
+            except OkerrNoKey:
+                self.log.debug('no such key: {}'.format(trypath))
+                pass
+
+        # if we are here, then no auth problem, but no key
+        raise OkerrNoKey('Failed all {} keypath: {}'.format(ntried, path))
 
     def keypath(self,path):
 
         if not self.url.endswith('/'):
             self.url+='/'
     
+        if not self.textid:
+            raise OkerrNoTextID('No textid. Cannot get keys.')
+            return None
+    
         url = self.url+'getkeyval/{}/{}'.format(self.textid,path)
 
         auth=None
+        
         if self.keyuser and self.keypath:
             auth=(self.keyuser,self.keypass)
+                
+        try:
+            r = requests.get(url, auth=auth)
+        except requests.exceptions.ConnectionError as e:
+            raise OkerrClientServer(str(e))
+        
+        # print(str(r.status_code))
         
-        r = requests.get(url, auth=auth)
         if r.status_code==200:
-            self.log.info('got keys OK')
+            data = json.loads(r.text)
+            return data
+
         elif r.status_code == 401:
-            self.log.error('Authentication required for getting keypath \'{}\' on project textid \'{}\', user keyuser/keypass'.format(path,self.textid))
+            raise OkerrKeyAuth('Authentication required for getting keypath \'{}\' on project textid \'{}\', keyuser \'{}\''.format(path,self.textid,self.keyuser))
             return None
+        elif r.status_code == 404:
+            raise OkerrNoKey('no key {}',path)
         else:
-            self.log.info('okerr getkeyval error {} \'{:.50}\' textid:{} {}'.\
-                format(r.status_code,r.text,self.textid,path))
+            self.log.error('okerr getkeyval error {} \'{:.50}\' textid:{} {}'.\
+                format(r.status_code,briefpage(r.text),self.textid,path))
+            raise OkerrClientServer('Server communication error {}: {:.50}'.format(r.status_code, briefpage(r.text)))
             return None
-        
-        data = json.loads(r.text)
-        return data
+        return None        
         
         
 
     def update(self,name,status,details=None,
-        method=None,tags=None,error=None):
+        method=None,tags=None,error=None, origkeypath=None, keypath=None):
+        
+        if not self.textid:
+            self.log.error('Do not update: no textid')
+            return
         
         # fix name
         if name.startswith(':') and self.prefix is not None:
@@ -149,11 +225,19 @@ class OkerrClient:
     
         url = self.url+'update'
         
-        self.log.debug("update: {}:{} = {} ({}) url: {}".format(self.textid,name,status,details, self.url))
+        self.log.debug(u"update: {}:{} = {} ({}) url: {}".format(self.textid,name,status,details, self.url))
 
         
+        if keypath is None:
+            keypath=''
+            
+        if origkeypath is None:
+            origkeypath=''
+
+
         payload={'textid': self.textid, 'name':name, 'status': str(status), 
-            'details': details, 'secret': self.secret, 'method': method, 'tags': ','.join(tags),'error': error}
+            'details': details, 'secret': self.secret, 'method': method, 'tags': ','.join(tags),'error': error,
+            'keypath': keypath, 'origkeypath': origkeypath}
 
         if self.secret:
             secretlog="[secret]"
@@ -162,27 +246,55 @@ class OkerrClient:
         start = time.time()
         
         
-        try:
-            r = requests.post(url, data=payload)
-            if r.status_code==200:
-                self.log.info('okerr send update OK textid:{}, {}={} {}'.\
-                    format(self.textid,name,status,secretlog))
+        if len(str(status))>20:
+            preview = str(status)[:18]+".."
+        else:
+            preview = str(status)
+            
+        
+        stop = False
+        success = False
+        
+        while not stop:        
+            try:
+                r = requests.post(url, data=payload)        
+                if r.status_code == 200:
+                    stop = True
+                    success = True
+                    self.log.info(u'okerr updated {} = {}'.\
+                        format(name,preview))
+                else:
+                    if self.retry:
+                        self.log.error('okerr update error ({}) textid:{}, {}={} {}'.\
+                            format(r.status_code,self.textid,name,status,secretlog))            
+                    else:
+                        raise OkerrClientServer('okerr update error {} \'{:.50}\' textid:{}, {}={} {}'.\
+                            format(r.status_code,briefpage(r.text),self.textid,name,status,secretlog))
+            
+                self.log.debug("Request to URL {}:".format(r.request.url))
+                self.log.debug(r.request.body)
+                
+            except requests.exceptions.ConnectionError as e:
+                self.log.error('okerr exception {} textid:{}, {}={} {}'.\
+                    format(str(e),self.textid,name,status,secretlog))
+            
+            if self.retry: 
+                if not stop:
+                    time.sleep(self.sleeptime)
             else:
-                self.log.info('okerr update error {} \'{:.50}\' textid:{}, {}={} {}'.\
-                    format(r.status_code,r.text,self.textid,name,status,secretlog))
-        
-            self.log.debug("Request to URL {}:".format(r.request.url))
-            self.log.debug(r.request.body)
+                # if no retries, simulate succeess
+                stop=True
+            
             
-        except requests.exceptions.ConnectionError as e:
-            self.log.info('okerr exception {} textid:{}, {}={} {}'.\
-                format(str(e),self.textid,name,status,secretlog))
      
         if r:
             self.log.debug(r.content)
         else:
             self.log.debug("no reply, check log")
         self.log.debug("took {} sec.".format(time.time() - start))
+    
+        return success
+
 
 
 def pid2name(pid):
