@@ -40,7 +40,7 @@ class TaskUptime(ProcTaskProcessor):
         bt = int(psutil.boot_time())        
         now = int(time.time())
         uptime = now-bt
-        ts.details = self.s2dhms(uptime)
+#        ts.details = self.s2dhms(uptime)
         
         return uptime
         
@@ -51,7 +51,7 @@ class TaskLoadAvg(ProcTaskProcessor):
     help = 'load average'    
     
     
-    defconf={
+    defargs={
         'period': '0'
     }
         
@@ -92,7 +92,7 @@ class TaskConnections(ProcTaskProcessor):
 
     parse_args = False
     
-#    defconf = {
+#    defargs = {
 #        'expr': 'proto=="tcp" and status=="LISTEN"',
 #        'fmt': '{proto} {ip}:{port} {status} ({basename})', 
 #        'ifmt': '{iname}:{proto}{port}', 
@@ -125,7 +125,7 @@ class TaskConnections(ProcTaskProcessor):
         first=True
         
         for p in psutil.process_iter():
-            try:
+            try:        
                 conns = p.connections()
                 if len(conns):
                     #print(p)
@@ -146,8 +146,9 @@ class TaskConnections(ProcTaskProcessor):
                                 
                         outdata.append(cs)
          
-            except psutil.AccessDenied:
+            except (psutil.AccessDenied, psutil.NoSuchProcess):
                 pass
+                
         return outdata
 
 TaskConnections('CONNECTIONS',ts.TaskSeq)
