@@ -10,30 +10,24 @@ import re
 import datetime
 import time
 import six
+import sys
+import socket
 
 from operator import itemgetter
 
 import okerrclient
-
-class NoMethodException(Exception):
-    pass
-
-class BadMethodException(Exception):
-    pass
-
-class BadInputException(Exception):
-    pass
-
-
+from okerrclient import OkerrExc, OkerrNoKey, OkerrNoTextID, OkerrKeyAuth, \
+    OkerrBadData, OkerrBadMethod
 
 #
 # Taskprocessor is parent class for processor, which implements sequence code
 # like 'DIR' or 'JDUMP'
 #
 
-class TaskProcessor:
+class TaskProcessor(object):
 
     _help='help stub'
+    help_suffix = None
     code = '<No Code>'
     
     store_argline=None # if set, whole argline stored in this key in args{} dict
@@ -42,7 +36,8 @@ class TaskProcessor:
 
     chapter = 'NO SPECIFIC CHAPTER'
 
-    defconf={}
+    defargs={}
+    tpconf={}
 
 
     #
@@ -57,25 +52,53 @@ class TaskProcessor:
     def help(self):
         return 'ZZZ' + self._help
 
+    @staticmethod
+    def cmd2str(cmd):
+        cmdstr=cmd['command']
+        for arg in cmd:
+            if arg !='command':
+                cmdstr += " {}={} ".format(arg,cmd[arg])
+        return cmdstr
+
+
     def fullhelp(self):
         
         msg = ""
         msg += self.code+'\t'+self.help+'\n'
-        if self.parse_args and self.defconf:                    
-            msg += '\tkey-arguments:\n';
-            for aname in sorted(self.defconf):
-                msg += '\t'+aname+'="'+self.defconf[aname]+'"\n'
+        
+        if self.tpconf:
+            msg += '\tconfiguration:\n'
+            for cname in sorted(self.tpconf):
+                msg += '\t{} (--tpconf {}:{}=...): {}\n'.format(cname,self.code,cname,self.tpconf[cname])
+            msg+='\n'
+        
+        if self.parse_args and self.defargs:                    
+            msg += '\tArguments:\n';
+            for aname in sorted(self.defargs):
+                msg += '\t'+aname+'='+repr(self.defargs[aname])+'\n'
         if self.store_argline:
-            msg+='\targument ({}), default: "{}"\n'.format(self.store_argline,self.defconf[self.store_argline])
+            msg+='\tparameter ({}), default: "{}"\n'.format(self.store_argline,self.defargs[self.store_argline])
+            
+        if self.help_suffix:
+            msg+=self.help_suffix
+            
         return msg
         
+    def tpconfset(self,k,v):
+        if k in self.tpconf and isinstance(self.tpconf[k],six.string_types):
+            self.tpconf[k]=v
+        elif k in self.tpconf and isinstance(self.tpconf[k],list):
+            self.tpconf[k].append(v)
+        else:
+            self.tsclass.oc.log.error('Task Processor \'{}\' has no configuraton key \'{}\''.format(self.code,k))            
+          
 
     def __init__(self, code, tsclass):
         self.code = code
         tsclass.addtproc(code,self)
    
     def parseargs(self, args):
-        ad = dict(self.defconf) # copy default dictionary
+        ad = dict(self.defargs) # copy default dictionary
         
         # set method
         ad['command']=self.code
@@ -91,7 +114,8 @@ class TaskProcessor:
                     else:
                         ad[v]=None
                 else:
-                    self.tsclass.oc.log.error('bad argument: {}'.format(args))
+                    self.tsclass.oc.log.error('bad argument: {}, try: okerr-client --help {}'.format(args, self.code))
+                    sys.exit(1)
 
         if self.store_cmdline:
             ad[self.store_cmdline] = args
@@ -122,6 +146,16 @@ class TaskSeq():
     method = None # method of indicator
     details = None # details
 
+
+    #
+    # origkeypath is what was on start (e.g. servers:srv1)
+    # keypath is dynamical part, e.g. df
+    # full keypath is combination: servers:srv1:df
+    #
+    
+    keypath = None
+    origkeypath = None
+
     tags=[]
     
     v = {}
@@ -133,10 +167,29 @@ class TaskSeq():
 
     _stop = False
 
+    steps = None # if not none, stop after this N of steps
+    
+    dump = None # method of debug dump 
+    dumpname = None
+    lastcmd = None
     
     def stop(self):
         self._stop = True
 
+    # taskseq.setsteps
+    def setsteps(self,steps):
+        self.steps=steps
+
+    # taskseq.setdump
+    def setdump(self,dump):
+        self.dump=dump
+
+    # taskseq.setdumpname
+    def setdumpname(self,dumpname):
+        self.dumpname=dumpname
+
+
+    
     #
     # TaskSeq.mkargs
     #
@@ -145,12 +198,16 @@ class TaskSeq():
     # 
     def mkargs(self,s):
         if isinstance(s,six.string_types):
-            m = shlex.split(s)[0]
+            try:
+                m = shlex.split(s)[0]
+            except ValueError:
+                raise ValueError('Cannot parse command: {}'.format(s))
+                return None
             #self.oc.log.debug('parsing command {}'.format(s))
             
             
             if not m in self.tp:
-                raise NoMethodException('No method '+m)
+                raise OkerrBadMethod('No method '+m)
                                    
             # MK ARGS
             args = self.tp[m].parseargs(s)
@@ -162,13 +219,13 @@ class TaskSeq():
             if not 'command' in s:
                 raise ValueError
             if not s['command'] in self.tp:
-                raise NoMethodException        
+                raise OkerrBadMethod('no method {}'.format(s['command']))        
                 
             # fill defargs
             code = s['command']
-            for k in self.tp[code].defconf:
+            for k in self.tp[code].defargs:
                 if not k in s:
-                    s[k]=self.tp[code].defconf[k]
+                    s[k]=self.tp[code].defargs[k]
             
             if self.tp[code].store_argline and 'argline' in s:
                 s[self.tp[code].store_argline] = s['argline']
@@ -201,20 +258,27 @@ class TaskSeq():
     # TaskSeq.__init__
     #
 
-    def __init__(self,iname,route,method=None):
+    def __init__(self,iname=None,route=None,method=None):
+        
+        if iname is None:
+            iname='noname'
+        
         self.iname = iname
+                        
+        self.dump = ['SEQDUMP']
         
         self.initvars()
 
         if route is not None:
             self.setseq(route)
 
-
         self.method=method
         self.tags=list()
         
-        # print("created. iname:{}".format(iname))
-
+        self.keypath = list()
+        self.origkeypath = list()
+        
+     
     def initvars(self):
         pass
   
@@ -223,27 +287,50 @@ class TaskSeq():
     # TaskSeq.fork
     # 
 
-    def fork(self,iname,route,method=None):
-        ts = TaskSeq(iname,route,method)
+    def fork(self,iname=None,route=None,method=None):
+        ts = TaskSeq(iname,route,method)        
+        
+        if iname is None:
+            iname = self.iname
+        if method is None:
+            method=self.method        
+            
+
         ts.iname = iname
-        ts.oc = self.oc
-        ts.method = self.method
+        ts.oc = self.oc        
+        ts.method = method
         ts.details = self.details
         ts.parent = self
         ts.tags = list(self.tags)
+        ts.keypath = self.keypath
+        ts.origkeypath = self.origkeypath
+        
+        ts.dump = self.dump
+        ts.dumpname = self.dumpname
+        ts.steps = self.steps
+        
         return ts
 
 
     def checkroute(self):
         for r in self.route:
             if not 'command' in r or not r['command'] in self.tp:
-                raise NoMethodException("No method '{}'".format(m))
+                raise OkerrBadMethod("No method '{}'".format(m))
 
         return True
 
     def setvar(self,name,value):
+        if(type(value) is str):
+            value = unicode(value,"utf8")
         self.v[name]=value
 
+
+    def getvar(self,name):
+        if name in self.v:
+            return self.v[name]
+        return None
+
+
     def getvars(self):
        v = self.v.copy()
        v['_name']=self.iname
@@ -291,12 +378,20 @@ class TaskSeq():
                 msg+=helpchapter(chapters,chapter)
             
         return msg
+
+
+    #
+    # taskseq.runmethod
+    #
+    def runcmd(self, cmd, data, args):
+        return self.tp[cmd].run(self,data,args)
+
        
    #
    # TaskSeq.run
    #
    
-    def run(self,data=None):
+    def run(self,data=None,steps=None):
         # print("run ({}) {}".deformat(self.iname,self.route))
 
         def strcmd(r):
@@ -305,9 +400,20 @@ class TaskSeq():
                 if k!='command':
                     arg+="{}={} ".format(k,v)    
             return "{} {}".format(r['command'],arg)
-                   
-        for r in self.fullroute:
-            self.route.pop(0)
+        
+        for r in self.fullroute:        
+            if self.steps is not None:                
+                if self.steps == 0:                
+                    if self.dumpname is None or self.dumpname == self.iname:
+                        for d in self.dump:
+                            self.runcmd(d,data,dict())
+                        self.stop()
+                else:                     
+                    self.steps -= 1
+    
+    
+       
+            self.lastcmd = self.route.pop(0)
     
             if self._stop:
                 self.oc.log.debug('sequence name:{} id:{} got stop signal'.format(self.iname, id(self)))
@@ -326,40 +432,71 @@ class TaskSeq():
                            
             # replace vars
             for argname,argval in r.items():
-                #print("replace arg {} = {}".format(argname, argval))
-                #print(vv)
+                # print("replace arg {} = {}".format(argname, argval))
+                # print(vv)
                 newval = string.Template(argval).safe_substitute(vv)
                 #print("newval: {}".format(newval))
                 r[argname] = newval
 
             
             if not cmd in self.tp:
-                raise NoMethodException                   
+                raise OkerrBadMethod('no method {}'.format(cmd))                   
                         
             # RUN task processor
-            data = self.tp[cmd].run(self,data,r)
+            #print("run, args: ",str(r))
+            #data = self.tp[cmd].run(self,data,r)
+            data = self.runcmd(cmd,data,r)
+            
             #self.oc.log.debug('output: {}'.format(data))
     
         # print('i: {}, data: {}'.format(self.iname,data))
         
         if self.iname is not None: 
             if data is not None:
+                       
                 self.oc.update(self.iname,data,details=self.details,
-                method=self.method,tags=self.tags,error=None)
+                method=self.method,tags=self.tags,error=None, origkeypath = ':'.join(self.origkeypath), keypath = ':'.join(self.keypath))
             else:
-                self.oc.log.info('do no update indicator {}, because data is None'.format(self.iname))
+                self.oc.log.debug('do no update indicator {}, because data is None'.format(self.iname))
         else:
 
-            self.oc.log.info('do not update indicator, because no indicator name')
+            self.oc.log.debug('do not update indicator, because no indicator name')
 
 
 
+class TaskAlias(TaskProcessor):
+    chapter='Alias'
+    
+    alias=[]
+    
+       
+    def fullhelp(self):
+        if sys.version_info.major == 2:
+            msg = super(TaskAlias, self).fullhelp()
+        else:
+            msg = super().fullhelp()
+        
+        msg += '\tAlias for: '+str(self.alias)+'\n'
+        return msg
+    
+    def run(self,ts,data,args):
+        print("ALIAS RUN")
+        for code in self.alias:
+            print("RUN CODE {}".format(code))
+            data = ts.tp[code].run(ts,data,args)
+
+        return data
+
+
 class GenTaskProcessor(TaskProcessor):
     chapter='General'
 
 class JSONTaskProcessor(TaskProcessor):
     chapter='JSON processing'
 
+class FormatTaskProcessor(TaskProcessor):
+    chapter='Formatting'
+
 
 class TaskMyName(GenTaskProcessor):
     help = 'returns self name'
@@ -380,7 +517,7 @@ class TaskSet(GenTaskProcessor):
     parse_args=False
     store_argline='set'
     
-    defconf= { 'set': ''}
+    defargs= { 'set': ''}
     
     help = 'Set variable'
     def run(self,ts,data,args):
@@ -395,29 +532,46 @@ class TaskSet(GenTaskProcessor):
         
         return data
     
-    @property
-    def help(self):
-        return 'returns {}'.format(self.code)
-
+   
 TaskSet('SET',TaskSeq)
 
 
+class TaskExport(GenTaskProcessor):
+    
+    parse_args=False
+    store_argline='exp'
+    
+    defargs= { 'exp': '' }
+    
+    help = 'Export data (string) or string element of dict (string) to sequence variable'
+    def run(self,ts,data,args):
+    
+        kv = re.split('\s*=\s*',args['exp'])
+        if len(kv)==1 and isinstance(data,six.string_types):
+            # this is string
+            ts.setvar(kv[0],data)
+        elif len(kv)==2 and isinstance(data,dict) and kv[1] in data:
+            ts.setvar(kv[0],data[kv[1]])
+        else:
+            ts.oc.log.error('export error. Exp: {}, Data: {}'.format(args['exp'],str(data)[:50]))
+                                        
+        return data
+    
+TaskExport('EXPORT',TaskSeq)
+
+
 class TaskTag(GenTaskProcessor):
     
     parse_args=False
     store_argline='tag'
     
-    defconf= { 'tag': ''}
+    defargs= { 'tag': ''}
     
     help = 'Set tag'
     def run(self,ts,data,args):
         ts.settag(args['tag'])        
         return data
     
-    @property
-    def help(self):
-        return 'returns {}'.format(self.code)
-
 TaskTag('TAG',TaskSeq)
 
 
@@ -433,17 +587,24 @@ TaskVersion('VERSION',TaskSeq)
 
 
 class TaskStr(GenTaskProcessor):
-    help = 'if str argument is set, returns str, else returns str(data)'
+    help = 'if str argument is set, returns str. Joins list by \n, otherwise returns str(data).'
  
     store_argline='str'
     parse_args=False
 
-    defconf = {'str': ''}
+    defargs = {'str': ''}
     def run(self,ts,data,args):
         if args['str']:
-            return args['str']
+            s = args['str']
         else:
-            return ' '.join(data)
+            try:       
+                s = "\n".join(data)
+            except:
+                s = str(data)
+    
+        # now, trim it
+        # s = s.strip('\r\n\t ')
+        return s
 
 TaskStr('STR',TaskSeq)
 
@@ -453,7 +614,7 @@ class TaskMethod(GenTaskProcessor):
     store_argline='method'
     parse_args=False
 
-    defconf = {'method': ''}
+    defargs = {'method': ''}
     def run(self,ts,data,args):
         #mw = args['method'].split(' ')
         mw = shlex.split(args['method'])
@@ -470,14 +631,15 @@ class TaskName(GenTaskProcessor):
     store_argline='name'
     parse_args=False
 
-    defconf = {'name': '$_name:noname'}
+    defargs = {'name': '$_name:noname'}
     def run(self,ts,data,args):
-        #vv = ts.getvars()
-        #if isinstance(data,dict):
-        #    vv.update(data) 
-        #ts.oc.log.debug("NAME: {}".format(args['name']))                
-        #ts.iname = string.Template(args['name']).safe_substitute(vv)
-        ts.iname = args['name']
+        vv = ts.getvars()
+        if isinstance(data,dict):
+            vv.update(data) 
+               
+        ts.oc.log.debug(u"NAME: {}".format(args['name']))                
+        ts.iname = string.Template(args['name']).safe_substitute(vv)
+        #ts.iname = args['name']
         
         return data
 
@@ -491,8 +653,8 @@ class TaskDetails(GenTaskProcessor):
     store_argline='format'
     parse_args=False
 
-    defconf = {
-        'format': '<no format given>',
+    defargs = {
+        'format': '',
     }
 
     def run(self,ts,data,args):
@@ -501,10 +663,18 @@ class TaskDetails(GenTaskProcessor):
         fmt=args['format']
 
         if data is None:
-            return fmt
+            ts.details = fmt
+            return None # can return data. anyway it's None
 
-        etpl = string.Template(fmt)            
-        ts.details = etpl.safe_substitute(data)            
+        if fmt:
+            if isinstance(data,dict):       
+                etpl = string.Template(fmt)            
+                ts.details = etpl.safe_substitute(data)            
+            else:
+                ts.details = fmt            
+             
+        else:
+            ts.details = str(data)
         
 
         # return same data 
@@ -512,6 +682,32 @@ class TaskDetails(GenTaskProcessor):
                 
 TaskDetails('DETAILS',TaskSeq)
 
+
+
+class DefaultProc(GenTaskProcessor):
+    help = 'Set default data if data is empty'
+
+    store_argline='def'
+    parse_args=False
+
+    defargs = {
+        'def': '',
+    }
+
+    def run(self,ts,data,args):
+        if not data:
+            try:
+                data = json.loads(args['def'])
+            except ValueError as e:
+                ts.oc.log.error("{}: not valid default JSON: {}".format(self.code, args['def']))
+                ts.stop()
+                return None
+                
+        return data
+                
+DefaultProc('DEFAULT', TaskSeq)        
+
+
 class TaskInt(GenTaskProcessor):
     help = 'throws away fractional part'
     def run(self,ts,data,args):
@@ -556,14 +752,14 @@ TaskNone('NONE',TaskSeq)
 
 class TaskSort(GenTaskProcessor):
     help = 'sort words or lines or lists'
-    defconf = {'num': '0', 'sep': ' ', 'empty': '0','field': ''}
+    defargs = {'num': '0', 'sep': ' ', 'empty': '0','field': ''}
         
     def run(self,ts,data,args):
         was_str=False
         sep = args['sep'].replace('\\n','\n')
 
         if data is None:
-            raise BadInputException('attempt to SORT None data')
+            raise OkerrBadData('attempt to SORT None data')
     
         # make sure we have list
         if isinstance(data,str):
@@ -606,11 +802,16 @@ class TaskSeqDump(GenTaskProcessor):
     def run(self,ts,data,args):
         datadumplen=100
     
-        print("Sequence ({}): {}".format(ts.iname,id(ts)))
+        print("Sequence name '{}': #{}".format(ts.iname,id(ts)), end=" ")
         if ts.parent is None:
             print("no parent")
         else:
-            print("Parent {}: {}".format(id(ts.parent),ts.parent.iname))
+            print("Parent '{}': #{}".format(ts.parent.iname, id(ts.parent)))
+        print("lastcmd:",TaskProcessor.cmd2str(ts.lastcmd))
+        print("route:")
+        for r in ts.route:
+            print ("\t{} ".format(TaskProcessor.cmd2str(r)))
+            
         print("details:",ts.details)
         print("method:",ts.method)
         print("tags:",ts.tags)
@@ -645,7 +846,7 @@ class TaskFromJSON(JSONTaskProcessor):
         except ValueError:
             ts.oc.log.error('Try to json-decode non json data: {}'.format(data))
             msg = '{}: Try to decode non-JSON data'.format(self.code, data)
-            raise BadInputException(msg)
+            raise OkerrBadData(msg)
             return None
         return data                
         
@@ -657,12 +858,21 @@ class TaskJSONDump(JSONTaskProcessor):
     help = 'dump data in pretty JSON format to stdout. (returns None)'
     def run(self,ts,data,args):
         print(json.dumps(data, sort_keys=True, indent=4, separators=(',', ': ')))
-        return None
+        return data
         
 TaskJSONDump('JDUMP',TaskSeq)
 
 class TaskMkSeq(GenTaskProcessor):
-    help = 'make sequence from JSON/keyval structure'
+    help = 'make sequence from JSON/keyval structure or string. or from path (like: "servers:serverNNN|conf:anyserver", will try first path, if no data, it will try second and so on)'
+
+    store_argline='path'
+    parse_args=False
+
+    origkeypath = None
+
+    defargs = {
+        'path': ''
+    }
         
     def mkroute(self,ts,data):
         route = list()
@@ -675,14 +885,15 @@ class TaskMkSeq(GenTaskProcessor):
                        
             cmd = args['command']
                                     
-#            for defarg in ts.tp[cmd].defconf:
+#            for defarg in ts.tp[cmd].defargs:
 #                if not defarg in e:
-#                    e[defarg]= ts.tp[cmd].defconf[defarg]
+#                    e[defarg]= ts.tp[cmd].defargs[defarg]
                    
             route.append(args)
         return route 
     
-    def launch(self, ts, data,v=None):
+    # mkseq.launch
+    def launch(self, ts, data,v=None,keypath=None):
         
 
         #name = string.Template(data['name']).safe_substitute(v)
@@ -699,44 +910,68 @@ class TaskMkSeq(GenTaskProcessor):
 
         # print("route:",route)
 
-        try:
-            newts = okerrclient.taskseq.TaskSeq(ts.iname, None, ts.method)
-            newts.setseq(route)
-                        
-        except okerrclient.taskseq.NoMethodException as e:
-            print(e)
-            sys.exit(1)
+        # newts = okerrclient.taskseq.TaskSeq(ts.iname, None, ts.method)
+        newts = ts.fork() 
         
+        newts.setseq(route)
+        if not self.origkeypath is None:
+            newts.origkeypath = self.origkeypath
+            newts.keypath = keypath
+                    
+                    
         newts.run()
 
 
-        
-    def run(self,ts,data,args,v=None):
+    # mkseq.run
+    def run(self,ts,data,args,v=None,keypath=None):
+    
+               
+            
+        if keypath is None:
+            keypath = list()
     
-        if data is None:
-            ts.oc.log.debug('will not MKSEQ because data is None')
-            return None
     
-        if v is None:
-            v=dict()
-        
         def hasdict(d):
             for sdname,subdata in d.items():
                 if isinstance(subdata,dict):
                     return True
             return False
             
+    
+        if data is None :
+            # print("no data")
+            if args['path']:
+                data,path = ts.oc.altkeypath(args['path'])
+                # everything is fine!
+                self.origkeypath = path.split(':')                
+                self.keypath=list()
+                    
+                                    
+                    
+            else:
+                ts.oc.log.debug('will not MKSEQ because data is None and no path given (or empty)')
+                return None
+        else:
+            # print("have data")
+            pass
+
+    
+        if v is None:
+            v=dict()        
         
         if hasdict(data):
             for sdname,subdata in data.items():
                 if isinstance(subdata,str):
                     v[sdname]=subdata    
+                    
                                 
             for sdname,subdata in data.items():
                 if isinstance(subdata,dict):
-                    self.run(ts, subdata, args, v)                  
+                    newkp = keypath[:]
+                    newkp.append(sdname)
+                    self.run(ts, subdata, args, v, newkp)                  
         else:
-            self.launch(ts,data,v)
+            self.launch(ts,data,v,keypath)
         
         # stop this sequence
         ts.stop()
@@ -749,7 +984,7 @@ TaskMkSeq('MKSEQ', TaskSeq)
 class TaskDateTime(GenTaskProcessor):
     help = 'Set date and time variables'
 
-    defconf = {
+    defargs = {
         'prefix': '_',
         'offset': '0'
     }
@@ -791,7 +1026,7 @@ class TaskKeyVal(GenTaskProcessor):
     store_argline='path'
     parse_args=False
 
-    defconf = {
+    defargs = {
         'path': '',
     }
 
@@ -799,8 +1034,145 @@ class TaskKeyVal(GenTaskProcessor):
         #print ("get keyval data from path: {}".format(args['path']))
         #print("args:", args)
         data = ts.oc.keypath(args['path'])
+    
+        if data is None:
+            ts.oc.log.error('Failed to get keyval. Maybe you forgot to create path \'{}\' in okerr web interface?'.format(args['path']))
+            ts.stop()
+
         return data
 
 TaskKeyVal('KEYVAL', TaskSeq)
 
+class TaskSave(GenTaskProcessor):
+    help = 'Save data to variable'
+
+    store_argline='varname'
+    parse_args=False
+
+    defargs = {
+        'varname': '',
+    }
+
+    def run(self,ts,data,args):               
+        ts.setvar(args['varname'], data)    
+        return data
+
+TaskSave('SAVE', TaskSeq)
+
+class TaskLoad(GenTaskProcessor):
+    help = 'Load data from variable'
+
+    store_argline='varname'
+    parse_args=False
+
+    defargs = {
+        'varname': '',
+    }
+
+    def run(self,ts,data,args):               
+        return ts.getvar(args['varname'])    
+
+TaskLoad('LOAD', TaskSeq)
+
+class TaskDHMS(FormatTaskProcessor):
+    help = 'convert number of seconds to DHMS string e.g. (12d 2h)'
+
+    #store_argline='varname'
+    #parse_args=False
+
+    defargs = {
+        'num': '2',
+        'field': '',
+        'destfield': ''
+    }
+
+    @staticmethod
+    def dhms_short(sec, sep=" ", num=2):
+        out=""
+        nn=0
+        t={'d': 86400,'h': 3600,'m': 60,'s': 1}
+        for k in sorted(t,key=t.__getitem__,reverse=True):
+            if sec>t[k]:
+                if nn == num:
+                    break
+                nn+=1
+                n = int(sec/t[k])
+                sec-=n*t[k]
+                out+="%d%s%s" % (n,k,sep)
+        return out.strip()
+    
+
+    def run(self,ts,data,args):
+    
+        if isinstance(data,list):
+            return map(lambda d: self.run(ts,d,args), data)
+            
+        if isinstance(data,dict):
+            field = args['field']                        
+            if field:
+                dst = args['destfield']
+                if not dst:
+                    dst = field
+                sec = int(float(data[field]))
+                data[dst] = TaskDHMS.dhms_short(sec, ' ', int(args['num']))
+                return data                         
+                   
+                   
+        sec = int(float(data)) # in case if data is string or fraction
+        return TaskDHMS.dhms_short(sec,' ', int(args['num']))
+        
+
+TaskDHMS('DHMS', TaskSeq)
+
+
+
+class TaskKMGT(FormatTaskProcessor):
+    help = 'Convert number (e.g. size of file) to string with suffix (e.g. 12M)'
+
+    defargs = {
+        'frac': '10',
+        'field': '',
+        'destfield': ''
+    }
+
+
+    @staticmethod
+    def kmgt_short(sz, frac=10):
+        t={
+            'K': pow(1024,1),
+            'M': pow(1024,2),
+            'G': pow(1024,3),
+            'T': pow(1024,4),
+            '': 1}
+            
+        for k in sorted(t,key=t.__getitem__,reverse=True):
+            if sz>t[k]:
+                n = int((float(sz)*frac / t[k]))/float(frac)
+                return "{}{}".format(n,k)
+
+    def run(self,ts,data,args):               
+
+        if isinstance(data,list):
+            return map(lambda d: self.run(ts,d,args), data)
+            
+        if isinstance(data,dict):
+            field = args['field']                        
+            if field:
+                dst = args['destfield']
+                if not dst:
+                    dst = field
+                sz = int(float(data[field]))
+                data[dst] = TaskKMGT.kmgt_short(sz, int(args['frac']))
+                return data                         
+
+        sz = int(float(data)) # in case if data is string or fra
+        return TaskKMGT.kmgt_short(sz,int(args['frac']))
+
+TaskKMGT('KMGT', TaskSeq)
+
+
+
+
+
+
 
